<?php

declare(strict_types=1);

namespace mirzaev\arming_bot\controllers;

// Files of the project
use mirzaev\arming_bot\controllers\core,
	mirzaev\arming_bot\models\categories,
	mirzaev\arming_bot\models\category,
	mirzaev\arming_bot\models\product;

/**
 * Controller of catalog
 *
 * @package mirzaev\arming_bot\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class catalog extends core
{
	/**
	 * Registry of errors
	 */
	protected array $errors = [
		'session' => [],
		'account' => [],
		'catalog' => []
	];

	/**
	 * Catalog
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 */
	public function index(array $parameters = []): ?string
	{
		if (!empty($parameters['categories']) && $parameters['categories'] !== ['/']) {
			// Переданы категории ["category1", "category2", "category3"] (иерархия)

			// Инициализация актуальной категории
			$category = end($parameters['categories']);

			if ($model = categories::category($category)) {
				// Найдена модель обработки актуальной категории

				if (method_exists($model, 'entries')) {
					// Найден метод поиска вхождений

					// Поиск категорий или товаров входящих в актуальную категорию
					$entries = $model::entries(
						category: $model->getId(),
						filter: 'v.deleted != true && v.hidden != true',
						amount: 30,
						errors: $this->errors['catalog']
					);

					// Объявление буферов категорий и товаров (важно - в единственном числе, по параметру из базы данных)
					$category = $product = [];

					foreach ($entries as $entry) {
						// Перебор вхождений

						// Запись массивов категорий и товаров ($category и $product) в буфер глобальной переменной шаблонизатора
						${$entry->_type}[] = $entry;
					}

					// Запись категорий из буфера в глобальную переменную шаблонизатора
					$this->view->categories = $category;

					// Запись товаров из буфера в глобальную переменную шаблонизатора
					$this->view->products = $product;
				}
			}
		} else {
			// Не переданы категории

			// Поиск категорий: "categories"
			// @todo сделать автоматический поиск "самой верхней" категории
			$this->view->categories = category::_read(
				filter: 'd.deleted != true && d.hidden != true',
				sort: 'd.position ASC, d.name ASC, d.created DESC',
				amount: 30,
				errors: $this->errors['catalog']
			);

			// Search for products
			/* $this->view->products = product::read(
				filter: 'd.deleted != true && d.hidden != true',
				sort: 'd.promoting ASC, d.position ASC, d.created DESC',
				amount: 6,
				errors: $this->errors['catalog']
			); */
		}

		if ($_SERVER['REQUEST_METHOD'] === 'GET') {
			// GET request

			// Exit (success)
			return $this->view->render('catalog/page.html');
		} else if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			// POST request

			// Initializing a response headers
			header('Content-Type: application/json');
			header('Content-Encoding: none');
			header('X-Accel-Buffering: no');

			// Initializing of the output buffer
			ob_start();

			// Generating the reponse
			echo json_encode(
				[
					'title' => $title ?? '',
					'html' => [
						'categories' => $this->view->render('catalog/elements/categories.html'),
						'products' => $this->view->render('catalog/elements/products/2columns.html')
					],
					'errors' => $this->errors
				]
			);

			// Initializing a response headers
			header('Content-Length: ' . ob_get_length());

			// Sending and deinitializing of the output buffer
			ob_end_flush();
			flush();

			// Exit (success)
			return null;
		}

		// Exit (fail)
		return null;
	}

	/**
	 * Search
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 */
	public function search(array $parameters = []): ?string
	{
		// Initializing of text fore search
		preg_match('/[\w\s]+/u', $parameters['text'] ?? '', $matches);
		$text = $matches[0] ?? null;

		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			// POST request

			// Search for products
			$this->view->products = isset($text) ? product::read(
				search: $text,
				filter: 'd.deleted != true && d.hidden != true',
				sort: 'd.position ASC, d.name ASC, d.created DESC',
				amount: 30,
				errors: $this->errors['catalog']
			) : [];

			// Initializing a response headers
			header('Content-Type: application/json');
			header('Content-Encoding: none');
			header('X-Accel-Buffering: no');

			// Initializing of the output buffer
			ob_start();

			// Generating the reponse
			echo json_encode(
				[
					'title' => $title ?? '',
					'html' => [
						'products' => $this->view->render('catalog/elements/products.html')
					],
					'errors' => $this->errors
				]
			);

			// Initializing a response headers
			header('Content-Length: ' . ob_get_length());

			// Sending and deinitializing of the output buffer
			ob_end_flush();
			flush();

			// Exit (success)
			return null;
		}

		// Exit (fail)
		return null;
	}

	/**
	 * Product
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 */
	public function product(array $parameters = []): ?string
	{
		// Initializing of text fore search
		preg_match('/[\d]+/', $parameters['id'] ?? '', $matches);
		$_key = $matches[0] ?? null;

		if (!empty($_key)) {
			// Received id of prouct (_key)

			if ($_SERVER['REQUEST_METHOD'] === 'POST') {
				// POST request

				// Search for products
				$product = product::read(
					filter: "d._key == \"$_key\" && d.deleted != true && d.hidden != true",
					sort: 'd.created DESC',
					amount: 1,
					return: '{id: d._key, title: d.title.ru, description: d.description.ru, cost: d.cost, weight: d.weight, dimensions: d.dimensions, brand: d.brand.ru, compatibility: d.compatibility.ru}',
					errors: $this->errors['catalog']
				)[0]?->getAll();

				if (!empty($product)) {
					// Found the product

					// Initializing buffer of images
					$images = [];

					foreach (
						glob(INDEX .
							DIRECTORY_SEPARATOR .
							'themes' .
							DIRECTORY_SEPARATOR .
							(THEME ?? 'default') .
							DIRECTORY_SEPARATOR .
							'images' .
							DIRECTORY_SEPARATOR .
							$_key .
							DIRECTORY_SEPARATOR .
							"*.{jpg,png,gif}", GLOB_BRACE) as $file
					) {
						// Iterate over images of the product

						// Write to buffer of images
						$images[] = "/images/$_key/" . basename($file);
					}

					$product = $product + ['images' => $images];
				}

				// Initializing a response headers
				header('Content-Type: application/json');
				header('Content-Encoding: none');
				header('X-Accel-Buffering: no');

				// Initializing of the output buffer
				ob_start();

				// Generating the reponse
				echo json_encode(
					[
						'product' => $product,
						'errors' => $this->errors
					]
				);

				// Initializing a response headers
				header('Content-Length: ' . ob_get_length());

				// Sending and deinitializing of the output buffer
				ob_end_flush();
				flush();

				// Exit (success)
				return null;
			}
		}

		// Exit (fail)
		return null;
	}
}
