<?php

declare(strict_types=1);

namespace mirzaev\arming_bot\controllers;

// Files of the project
use mirzaev\arming_bot\controllers\core,
	mirzaev\arming_bot\models\account;

/**
 * Controller of session
 *
 * @package mirzaev\arming_bot\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class session extends core
{
	/**
	 * Registry of errors
	 */
	protected array $errors = [
		'session' => [],
		'account' => []
	];

	/**
	 * Connect session to the telegram account
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 */
	public function telegram(array $parameters = []): ?string
	{
		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			// POST request

			if ($connected = isset($this->account)) {
				// Found the account

				// Initializing language of the account
				$language = $this->account->language;
			} else {
				// Not found the account

				if (count($parameters) > 1 && isset($parameters['hash'])) {

					$buffer = $parameters;

					unset($buffer['authentication'], $buffer['hash']);
					ksort($buffer);

					$prepared = [];
					foreach ($buffer as $key => $value) {
						if (is_array($value)) {
							$prepared[] = $key . '=' . json_encode($value, JSON_UNESCAPED_UNICODE);
						} else {
							$prepared[] = $key . '=' . $value;
						}
					}

					$key = hash_hmac('sha256', require(SETTINGS . DIRECTORY_SEPARATOR . 'key.php'), 'WebAppData', true);
					$hash = bin2hex(hash_hmac('sha256', implode(PHP_EOL, $prepared), $key, true));

					if (hash_equals($hash, $parameters['hash'])) {
						// Data confirmed (according to telegram documentation)

						if (time() - $parameters['auth_date'] < 86400) {
							// Authorization date less than 1 day ago

							// Initializing data of the account
							$data = json_decode($parameters['user']);

							// Initializing of the account
							$account = account::initialization(
								$data->id,
								[
									'id' => $data->id,
									'name' => [
										'first' => $data->first_name,
										'last' => $data->last_name
									],
									'domain' => $data->username,
									'language' => $data->language_code,
									'messages' => $data->allows_write_to_pm
								],
								$this->errors['account']
							);

							if ($account instanceof account) {
								// Initialized the account

								// Connecting the account to the session
								$connected = $this->session->connect($account, $this->errors['session']);

								// Initializing language of the account
								$language = $account->language;
							}
						}
					}
				}
			}

			// Initializing a response headers
			header('Content-Type: application/json');
			header('Content-Encoding: none');
			header('X-Accel-Buffering: no');

			// Initializing of the output buffer
			ob_start();

			// Generating the reponse
			echo json_encode(
				[
					'connected' => (bool) $connected,
					'language' => $language ?? null,
					'errors' => $this->errors
				]
			);

			// Initializing a response headers
			header('Content-Length: ' . ob_get_length());

			// Sending and deinitializing of the output buffer
			ob_end_flush();
			flush();

			// Exit (success)
			return null;
		}

		// Exit (fail)
		return null;
	}
}
