<?php

declare(strict_types=1);

namespace mirzaev\arming_bot\models;

// Files of the project
use mirzaev\arming_bot\models\core,
	mirzaev\arming_bot\models\traits\status,
	mirzaev\arming_bot\models\traits\document as arangodb_document_trait,
	mirzaev\arming_bot\models\interfaces\document as arangodb_document_interface;

// Framework for ArangoDB
use mirzaev\arangodb\collection,
	mirzaev\arangodb\document;

// Library для ArangoDB
use ArangoDBClient\Document as _document;

// Framework for Telegram
use Zanzara\Telegram\Type\User as telegram;

// Built-in libraries
use exception;

/**
 * Model of an account
 *
 * @package mirzaev\arming_bot\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class account extends core implements arangodb_document_interface
{
	use status, arangodb_document_trait;

	/**
	 * Name of the collection in ArangoDB
	 */
	final public const string COLLECTION = 'account';

	/**
	 * Инициализация
	 *
	 * @param int $id Идентификатор Telegram
	 * @param telegram|array|null $registration Данные для регистрация, если аккаунт не найден
	 * @param array &$errors Registry of errors
	 *
	 * @return static|null Объект аккаунта, если найден
	 */
	public static function initialization(int $id, telegram|array|null $registration = null, array &$errors = []): static|null
	{
		try {
			if (collection::init(core::$arangodb->session, self::COLLECTION)) {
				if ($document = collection::search(core::$arangodb->session, sprintf("FOR d IN %s FILTER d.id == %u RETURN d", self::COLLECTION, $id))) {
					// Найден аккаунт

					// Инициализация объекта аккаунта
					$account = new static;

					// Запись инстанции документа в объект
					$account->document = $document;

					// Возврат (успех)
					return $account;
				} else if ($registration) {
					// Не найден аккаунт и запрошена его регистрация

					// Создание аккаунта
					document::write(
						core::$arangodb->session,
						self::COLLECTION,
						(is_array($registration)
							? $registration :
							[
								'id' => $registration->getId(),
								'name' => [
									'first' => $registration->getFirstName(),
									'last' => $registration->getLastName()
								],
								'domain' => $registration->getUsername(),
								'robot' => $registration->isBot(),
								'banned' => false,
								'tester' => false,
								'developer' => false,
								'access' => [
									'settings' => false
								],
								'menus' => [
									'attachments' => $registration->getAddedToAttachmentMenu()
								],
								'messages' => true,
								'groups' => [
									'join' => $registration->getCanJoinGroups(),
									'messages' => $registration->getCanReadAllGroupMessages()
								],
								'premium' => $registration->isPremium(),
								'language' => $registration->getLanguageCode(),
								'queries' => [
									'inline' => $registration->getSupportsInlineQueries()
								]
							]) + [
							'version' => ROBOT_VERSION,
							'active' => true
						]
					);

					// Инициализация (без регистрации)
					return static::initialization($id, errors: $errors);
				} else throw new exception('Account not found');
			} else throw new exception('Failed to initialize document collection: ' . self::COLLECTION);
		} catch (exception $e) {
			// Write to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Выход (провал)
		return null;
	}
}
