<?php

declare(strict_types=1);

namespace mirzaev\arming_bot\models;

// Files of the project
use mirzaev\arming_bot\models\core,
	mirzaev\arming_bot\models\traits\document as arangodb_document_trait,
	mirzaev\arming_bot\models\interfaces\document as arangodb_document_interface;

// Framework for ArangoDB
use mirzaev\arangodb\collection,
	mirzaev\arangodb\document;

// Library для ArangoDB
use ArangoDBClient\Document as _document;

// Built-in libraries
use exception;

/**
 * Model of settings
 *
 * @package mirzaev\arming_bot\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class settings extends core implements arangodb_document_interface
{
	use arangodb_document_trait;

	/**
	 * Name of the collection in ArangoDB
	 */
	final public const string COLLECTION = 'settings';

	/**
	 * Search for active settings
	 *
	 * @param array|null $create Данные для создания, если настройки не найдены
	 * @param array &$errors Registry of errors
	 *
	 * @return static|null Object implements an instance of settngs document from ArangoDB
	 */
	public static function active(array|null $create = null, array &$errors = []): static|null
	{
		try {
			if (collection::init(core::$arangodb->session, self::COLLECTION)) {
				if ($document = collection::search(core::$arangodb->session, sprintf("FOR d IN %s FILTER d.status == 'active' SORT d.updated DESC LIMIT 1 RETURN d", self::COLLECTION))) {
					// Найдены активные настройки

					// Инициализация объекта настроек
					$settings = new static;

					// Запись инстанции документа в объект
					$settings->document = $document;

					// Возврат (успех)
					return $settings;
				} else if ($create) {
					// Не найдены активные настройки и запрошено создание

					// Создание настроек
					document::write(core::$arangodb->session, self::COLLECTION, ['status' => 'active'] + $create);

					// Инициализация (без создания)
					return static::active(errors: $errors);
				} else throw new exception('Settings not found');
			} else throw new exception('Failed to initialize document collection: ' . self::COLLECTION);
		} catch (exception $e) {
			// Write to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Выход (провал)
		return null;
	}
}
