"use strict";

// Import dependencies
import("/js/core.js").then(() =>
	import("/js/damper.js").then(() => {
		import("/js/telegram.js").then(() => {
			import("/js/hotline.js").then(() => {
				const dependencies = setInterval(() => {
					console.log(typeof core, typeof core.damper, typeof core.telegram, typeof core.hotline);
					if (
						typeof core === "function" &&
						typeof core.damper === "function" &&
						typeof core.telegram === "function" &&
						typeof core.hotline === "function"
					) {
						clearInterval(dependencies);
						clearTimeout(timeout);
						initialization();
					}
				}, 10);
				const timeout = setTimeout(() => clearInterval(dependencies), 5000);

				function initialization() {
					if (typeof core.catalog === "undefined") {
						// Not initialized

						// Write to the core
						core.catalog = class catalog {
							/**
							 * Current position in hierarchy of the categories
							 */
							static categories = [];

							/**
							 * Select a category (interface)
							 *
							 * @param {HTMLElement} button Button of category <a>
							 * @param {bool} clean Clear search bar?
							 * @param {bool} force Ignore the damper?
							 *
							 * @return {void}
							 */
							static category(button, clean = true, force = false) {
								// Initialize of the new category name
								const category = button.getAttribute("data-category-name");

								this._category(category, clean, force);
							}

							/**
							 * Select a category (damper)
							 *
							 * @param {HTMLElement} button Button of category <a>
							 * @param {bool} clean Clear search bar?
							 * @param {bool} force Ignore the damper?
							 *
							 * @return {void}
							 */
							static _category = core.damper(
								(...variables) => this.__category(...variables),
								400,
								2,
							);

							/**
							 * Select a category (system)
							 *
							 * @param {HTMLElement} button Button of category <a>
							 * @param {bool} clean Clear search bar?
							 *
							 * @return {Promise} Request to the server
							 */
							static __category(category = "", clean = true) {
								if (typeof category === "string") {
									//

									let urn;
									if (category === "/" || category === "") urn = "/";
									else {urn = this.categories.length > 0
											? `/${this.categories.join("/")}/${category}`
											: `/${category}`;}

									return core.request(urn)
										.then((json) => {
											if (
												json.errors !== null &&
												typeof json.errors === "object" &&
												json.errors.length > 0
											) {
												// Errors received
											} else {
												// Errors not received

												if (clean) {
													// Clearing the search bar
													const search = core.main.querySelector(
														'search[data-section="search"]>input',
													);
													if (search instanceof HTMLElement) search.value = "";
												}

												// Write the category to position in the categories hierarchy
												if (category !== "/" && category !== "") {
													this.categories.push(category);
												}

												if (
													typeof json.title === "string" &&
													json.title.length > 0
												) {
													// Received the page title

													// Initialize a link to the categories list
													const title = core.main.getElementsByTagName("h2")[0];

													// Write the title
													title.innerText = json.title;
												}

												if (
													typeof json.html.categories === "string" &&
													json.html.categories.length > 0
												) {
													// Received categories (reinitialization of the categories)

													const categories = core.main.querySelector(
														'section[data-catalog-type="categories"]',
													);

													if (categories instanceof HTMLElement) {
														// Found list of categories

														categories.outerHTML = json.html.categories;
													} else {
														// Not found list of categories

														const element = document.createElement("section");

														const search = core.main.querySelector(
															'search[data-section="search"]',
														);

														if (search instanceof HTMLElement) {
															core.main.insertBefore(
																element,
																search.nextSibling,
															);

															element.outerHTML = json.html.categories;
														}
													}
												} else {
													// Not received categories (deinitialization of the categories)

													const categories = core.main.querySelector(
														'section[data-catalog-type="categories"',
													);
													if (categories instanceof HTMLElement) {
														categories.remove();
													}
												}

												if (
													typeof json.html.products === "string" &&
													json.html.products.length > 0
												) {
													// Received products (reinitialization of the products)

													const products = core.main.querySelector(
														'section[data-catalog-type="products"]',
													);

													if (products instanceof HTMLElement) {
														// Found list of products

														products.outerHTML = json.html.products;
													} else {
														// Not found list of products

														const element = document.createElement("section");

														const categories = core.main.querySelector(
															'section[data-catalog-type="categories"]',
														);

														if (categories instanceof HTMLElement) {
															core.main.insertBefore(
																element,
																categories.nextSibling,
															);

															element.outerHTML = json.html.products;
														} else {
															const search = core.main.querySelector(
																'search[data-section="search"]',
															);

															if (search instanceof HTMLElement) {
																core.main.insertBefore(
																	element,
																	search.nextSibling,
																);

																element.outerHTML = json.html.products;
															}
														}
													}
												} else {
													// Not received products (deinitialization of the products)

													const products = core.main.querySelector(
														'section[data-catalog-type="products"',
													);
													if (products instanceof HTMLElement) {
														products
															.remove();
													}
												}
											}
										});
								}
							}

							/**
							 * Select a category (interface)
							 *
							 * @param {Event} event Event (keyup)
							 * @param {HTMLElement} element Search bar <input>
							 * @param {bool} force Ignore the damper?
							 *
							 * @return {void}
							 */
							static search(event, element, force = false) {
								element.classList.remove("error");

								if (element.innerText.length === 1) {
									return;
								} else if (event.keyCode === 13) {
									// Button: "enter"

									element.setAttribute("disabled", true);

									this.__search(element);
								} else {
									// Button: any

									this._search(element, force);
								}
							}

							/**
							 * Search in the catalog (damper)
							 *
							 * @param {HTMLElement} button Button of category <a>
							 * @param {bool} clean Clear search bar?
							 * @param {bool} force Ignore the damper?
							 *
							 * @return {void}
							 */
							static _search = core.damper(
								(...variables) => this.__search(...variables),
								1400,
								2,
							);

							/**
							 * Search in the catalog (system)
							 *
							 * @param {HTMLElement} element Search bar <input>
							 *
							 * @return {Promise} Request to the server
							 *
							 * @todo add animations of errors
							 */
							static __search(element) {
								// Deinitialization of position in the categories hierarchy
								this.categories = [];

								return this.__category("/", false)
									.then(function () {
										core.request("/search", `text=${element.value}`)
											.then((json) => {
												element.removeAttribute("disabled");
												element.focus();

												if (
													json.errors !== null &&
													typeof json.errors === "object" &&
													json.errors.length > 0
												) {
													// Errors received

													element.classList.add("error");
												} else {
													// Errors not received

													if (
														typeof json.title === "string" &&
														json.title.length > 0
													) {
														// Received the page title

														// Initialize a link to the categories list
														const title =
															core.main.getElementsByTagName("h2")[0];

														// Write the title
														title.innerText = json.title;
													}

													// Deinitialization of the categories
													const categories = core.main.querySelector(
														'section[data-catalog-type="categories"]',
													);
													// if (categories instanceof HTMLElement) categories.remove();

													if (
														typeof json.html.products === "string" &&
														json.html.products.length > 0
													) {
														// Received products (reinitialization of the products)

														const products = core.main.querySelector(
															'section[data-catalog-type="products"]',
														);

														if (products instanceof HTMLElement) {
															// Found list of products

															products.outerHTML = json.html.products;
														} else {
															// Not found list of products

															const element = document.createElement("section");

															const categories = core.main.querySelector(
																'section[data-catalog-type="categories"',
															);

															if (categories instanceof HTMLElement) {
																core.main.insertBefore(
																	element,
																	categories.nextSibling,
																);

																element.outerHTML = json.html.products;
															} else {
																const search = core.main.querySelector(
																	'search[data-section="search"]',
																);

																if (search instanceof HTMLElement) {
																	core.main.insertBefore(
																		element,
																		search.nextSibling,
																	);

																	element.outerHTML = json.html.products;
																}
															}
														}
													} else {
														// Not received products (deinitialization of the products)

														const products = core.main.querySelector(
															'section[data-catalog-type="products"]',
														);
														if (products instanceof HTMLElement) {
															products.remove();
														}
													}
												}
											});
									});
							}

							/**
							 * Open product card (interface)
							 *
							 * @param {string} id Identifier of a product
							 * @param {bool} force Ignore the damper?
							 *
							 * @return {void}
							 */
							static product(id, force = false) {
								this._product(id, force);
							}

							/**
							 * Open product card (damper)
							 *
							 * @param {string} id Identifier of a product
							 * @param {bool} force Ignore the damper?
							 *
							 * @return {void}
							 */
							static _product = core.damper(
								(...variables) => this.__product(...variables),
								400,
								1,
							);

							/**
							 * Open product card (system)
							 *
							 * @param {string} id Identifier of a product
							 *
							 * @return {Promise} Request to the server
							 */
							static __product(id) {
								if (typeof id === "number") {
									//

									return core.request(`/product/${id}`)
										.then((json) => {
											if (
												json.errors !== null &&
												typeof json.errors === "object" &&
												json.errors.length > 0
											) {
												// Errors received
											} else {
												// Errors not received

												if (
													json.product !== null &&
													typeof json.product === "object"
												) {
													// Received data of the product

													// Deinitializing of the old winow
													const old = document.getElementById("window");
													if (old instanceof HTMLElement) old.remove();

													const wrap = document.createElement("section");
													wrap.setAttribute("id", "window");

													const card = document.createElement("div");
													// card.classList.add("product", "card");
													card.classList.add("card", "unselectable");

													const h3 = document.createElement("h3");
													h3.setAttribute("title", json.product.id);

													const title = document.createElement("span");
													title.classList.add("title");
													title.innerText = json.product.title;

													const brand = document.createElement("small");
													brand.classList.add("brand");
													brand.innerText = json.product.brand;

													const images = document.createElement("div");
													images.classList.add("images", "unselectable");

													for (const uri of json.product.images) {
														const image = document.createElement("img");
														image.setAttribute("src", uri);
														image.setAttribute("ondragstart", "return false;");

														const button = core.telegram.api.isVisible;

														const open = (event) => {
															if (event.target === from) {
																if (typeof images.hotline === "object") {
																	if (images.hotline.moving) return;
																	images.hotline.stop();
																}

																image.classList.add("extend");

																if (button) core.telegram.api.MainButton.hide();

																setTimeout(() => {
																	image.addEventListener("click", close);
																	image.addEventListener("touch", close);
																}, 300);
																image.removeEventListener("mouseup", open);
																image.removeEventListener("touchend", open);
															}
														};

														const close = () => {
															if (typeof images.hotline === "object") {
																images.hotline.start();
															}

															image.classList.remove("extend");

															if (button) core.telegram.api.MainButton.show();

															image.removeEventListener("click", close);
															image.removeEventListener("touch", close);
															image.addEventListener("mousedown", start);
															image.addEventListener("touchstart", start);
														};

														const start = (event) => {
															if (
																event.type === "touchstart" ||
																event.button === 0
															) {
																image.removeEventListener("mousedown", start);
																image.removeEventListener("touchstart", start);
																image.addEventListener("mouseup", open);
																image.addEventListener("touchend", open);
															}
														};

														image.addEventListener("mousedown", start);
														image.addEventListener("touchstart", start);

														images.append(image);
													}

													const description = document.createElement("p");
													description.classList.add("description");
													description.innerText = json.product.description;

													const compatibility = document.createElement("p");
													compatibility.classList.add("compatibility");
													compatibility.innerText = json.product.compatibility;

													const footer = document.createElement("div");
													footer.classList.add("footer");
													footer.classList.add("footer");

													const dimensions = document.createElement("small");
													dimensions.classList.add("dimensions");
													dimensions.innerText = json.product.dimensions.x +
														"x" +
														json.product.dimensions.y + "x" +
														json.product.dimensions.z;

													const weight = document.createElement("small");
													weight.classList.add("weight");
													weight.innerText = json.product.weight + "г";

													const cost = document.createElement("p");
													cost.classList.add("cost");
													cost.innerText = json.product.cost + "р";
	
													h3.append(title);
													h3.append(brand);
													card.append(h3);
													card.append(images);
													card.append(description);
													card.append(compatibility);
													footer.append(dimensions);
													footer.append(weight);
													footer.append(cost);
													card.append(footer);
													wrap.append(card);
													core.main.append(wrap);

													let width = 0;
													let buffer;
													[...images.children].forEach((child) =>
														width += child.offsetWidth + (isNaN(
																buffer = parseFloat(
																	getComputedStyle(child).marginRight,
																),
															)
															? 0
															: buffer)
													);

													history.pushState(
														{ product_card: json.product.id },
														json.product.title,
													);

													// блокировка закрытия карточки
													let from;
													const _from = (event) => from = event.target;
													wrap.addEventListener("mousedown", _from);
													wrap.addEventListener("touchstart", _from);

													const remove = () => {
														wrap.remove();
														wrap.removeEventListener("mousedown", _from);
														wrap.removeEventListener("touchstart", _from);
														document.removeEventListener("click", close);
														document.removeEventListener("touch", close);
														window.removeEventListener("popstate", remove);
													};

													const close = (event) => {
														if (
															from === wrap &&
															!card.contains(event.target) &&
															!!card &&
															!!(card.offsetWidth ||
																card.offsetHeight ||
																card.getClientRects().length)
														) {
															remove();
														}

														from = undefined;
													};

													document.addEventListener("click", close);
													document.addEventListener("touch", close);
													window.addEventListener("popstate", remove);

													if (width > card.offsetWidth) {
														images.hotline = new core.hotline(
															json.product.id,
															images,
														);
														images.hotline.step = -0.3;
														images.hotline.wheel = true;
														images.hotline.touch = true;
														images.hotline.start();
													}
												}
											}
										});
								}
							}
						};
					}
				}
			});
		});
	})
);
