<?php

declare(strict_types=1);

namespace mirzaev\arming_bot;

// Files of the project
use mirzaev\arming_bot\controllers\core as controller,
	mirzaev\arming_bot\models\core as model,
	mirzaev\arming_bot\models\chat;

// Фреймворк Telegram
use Zanzara\Zanzara,
	Zanzara\Context,
	Zanzara\Config;

/* ini_set('error_reporting', E_ALL);
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1); */

// Версия робота
define('ROBOT_VERSION', '1.0.0');

// Путь до настроек
define('SETTINGS', __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'settings');

// Путь до хранилища
define('STORAGE', __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'storage');

// Файл в формате xlsx с примером excel-документа для импорта каталога
define('CATALOG_EXAMPLE', STORAGE . DIRECTORY_SEPARATOR . 'example.xlsx');

// Файл в формате xlsx для импорта каталога
define('CATALOG_IMPORT', STORAGE . DIRECTORY_SEPARATOR . 'import.xlsx');

// Ключ чат-робота Telegram
define('KEY', require(SETTINGS . DIRECTORY_SEPARATOR . 'key.php'));

// Инициализация библиотек
require __DIR__ . DIRECTORY_SEPARATOR
	. '..' . DIRECTORY_SEPARATOR
	. '..' . DIRECTORY_SEPARATOR
	. '..' . DIRECTORY_SEPARATOR
	. '..' . DIRECTORY_SEPARATOR
	. 'vendor' . DIRECTORY_SEPARATOR
	. 'autoload.php';

// Инициализация ядра контроллеров MINIMAL
new controller(false);

// Инициализация ядра моделей MINIMAL
new model(true);

$config = new Config();
$config->setParseMode(Config::PARSE_MODE_MARKDOWN);
$config->useReactFileSystem(true);

$bot = new Zanzara(KEY, $config);

/* $bot->onUpdate(function (Context $ctx): void {
	var_dump($ctx->getMessage()->getWebAppData());
	var_dump($ctx->getEffectiveUser() );
}); */

$bot->onCommand('start', fn($ctx) => chat::start($ctx));
$bot->onCommand('contacts', fn($ctx) => chat::contacts($ctx));
$bot->onCommand('company', fn($ctx) => chat::company($ctx));
$bot->onCommand('community', fn($ctx) => chat::community($ctx));
$bot->onCommand('settings', fn($ctx) => chat::settings($ctx));

$bot->onText('💬 Контакты', fn($ctx) => chat::contacts($ctx));
$bot->onText('🏛️ О компании', fn($ctx) => chat::company($ctx));
$bot->onText('🎯 Сообщество', fn($ctx) => chat::community($ctx));
$bot->onText('⚙️  Настройки', fn($ctx) => chat::settings($ctx));

$bot->onCbQueryData(['mail'], fn($ctx) => chat::_mail($ctx));
$bot->onCbQueryData(['import_request'], fn($ctx) => chat::import_request($ctx));
$bot->onCbQueryData(['tuning'], fn($ctx) => chat::tuning($ctx));
$bot->onCbQueryData(['brands'], fn($ctx) => chat::brands($ctx));

// Инициализация middleware с обработкой аккаунта
$bot->middleware([chat::class, "account"]);

// Инициализация middleware с обработкой технических работ разных уровней
$bot->middleware([chat::class, "suspension"]);

// Запуск чат-робота
$bot->run();
